@@ -2,10 +2,9 @@
 
 import logging
 from collections import OrderedDict, defaultdict, deque
-from typing import Union
+from typing import Union, List
 
 import iribaker
-import rdflib
 from rdflib import Graph
 from rdflib.namespace import DC, DCTERMS, RDF, XSD
 from rdflib.term import IdentifiedNode, Literal, URIRef
@@ -24,6 +23,7 @@ from .const import (
 )
 from .utils import delete_accents, delete_diacritics, natural_sort
 
+
 class NifContext:
     pass
 
@@ -105,12 +105,14 @@ class NifString(NifBase):
         beginIndex: Union[Literal, int] = None,
         endIndex: Union[Literal, int] = None,
         referenceContext: NifContext = None,
+        graph: Graph = None,
     ):
         self.set_URIScheme(URIScheme)
         self.set_referenceContext(referenceContext)
         self.set_beginIndex(beginIndex)
         self.set_endIndex(endIndex)
         self.set_uri(uri)
+        self.set_graph(graph)
 
     def __eq__(self, other):
         return (
@@ -128,8 +130,8 @@ class NifString(NifBase):
         """
         if self._beginIndex is not None:
             return int(self._beginIndex)
-        elif self.referenceContext.in_memory_graph is not None:
-            for item in self.referenceContext.in_memory_graph.objects(subject=self.uri, predicate=NIF.beginIndex):
+        elif self.graph is not None:
+            for item in self.graph.objects(subject=self.uri, predicate=NIF.beginIndex):
                 return int(item)
         else:
             return None
@@ -141,8 +143,8 @@ class NifString(NifBase):
         """
         if self._endIndex is not None:
             return int(self._endIndex)
-        elif self.referenceContext.in_memory_graph is not None:
-            for item in self.referenceContext.in_memory_graph.objects(subject=self.uri, predicate=NIF.endIndex):
+        elif self.graph is not None:
+            for item in self.graph.objects(subject=self.uri, predicate=NIF.endIndex):
                 return int(item)
         else:
             return None
@@ -206,17 +208,32 @@ class NifString(NifBase):
         """
         if uri is not None:
             if not isinstance(uri, URIRef):
+                if isinstance(self, NifContext):
+                    obj_str = "&nif=context"
+                elif isinstance(self, NifContextCollection):
+                    obj_str = "&nif=collection"
+                elif isinstance(self, NifPage):
+                    obj_str = "&nif=page"
+                elif isinstance(self, NifParagraph):
+                    obj_str = "&nif=paragraph"
+                elif isinstance(self, NifSentence):
+                    obj_str = "&nif=sentence"
+                elif isinstance(self, NifPhrase):
+                    obj_str = "&nif=phrase"
+                elif isinstance(self, NifWord):
+                    obj_str = "&nif=word"
                 if not isinstance(self, NifContext):
-                    uri_format = OffsetBasedString
-                    if uri_format == OffsetBasedString:
+                    uri = uri.replace("&nif=context", "")
+                    if self.URIScheme == OffsetBasedString:
                         uri = (
                             uri
-                            + "#offset_"
+                            + obj_str
+                            + "_"
                             + str(self.beginIndex)
                             + "_"
                             + str(self.endIndex)
                         )
-                    elif uri_format == RFC5147String:
+                    elif self.URIScheme == RFC5147String:
                         uri = (
                             uri
                             + "#char="
@@ -274,6 +291,9 @@ class NifString(NifBase):
         """
         self._URIScheme = URIScheme
 
+    def set_graph(self, graph: Graph = None):
+        self.graph = graph
+
     def triples(self):
         """
         Generates all the triples
@@ -291,25 +311,6 @@ class NifString(NifBase):
             if self._referenceContext is not None:
                 yield (self.uri, NIF.referenceContext, self._referenceContext.uri)
 
-    def load(
-        self,
-        graph: Graph = None,
-        referenceContext: NifContext = None,
-        uri: URIRef = None,
-    ):
-        """
-        Load a nif:String from a graph (based on uri)
-        """
-        self.set_uri(uri)
-        self.set_referenceContext(referenceContext)
-        # for s, p, o in graph.triples((uri, NIF.beginIndex, None)):
-        #     self.set_beginIndex(o)
-        # for s, p, o in graph.triples((uri, NIF.endIndex, None)):
-        #     self.set_endIndex(o)
-        # for s, p, o in graph.triples((uri, NIF.anchorOf, None)):
-        #     self.set_anchorOf(o)
-        return self
-
 
 class NifContext(NifString):
     """
@@ -337,6 +338,8 @@ class NifContext(NifString):
         predLang: URIRef = None,
         isString: Union[Literal, str] = None,
         metadata: dict = None,
+        lexicon: URIRef = None,
+        graph: Graph = None,
     ):
         super().__init__(
             URIScheme=URIScheme,
@@ -345,15 +348,16 @@ class NifContext(NifString):
             endIndex=len(isString) if isString is not None else None,
             referenceContext=self,
         )
-        self.set_sourceUrl(sourceUrl)
-        self.set_predLang(predLang)
-        self.set_isString(isString)
-        self.set_metadata(metadata)
         self.set_Sentences(None)
         self.set_Paragraphs(None)
         self.set_Pages(None)
         self.set_Phrases(None)
-        self.set_in_memory_graph(None)
+        self.set_graph(graph)
+        self.set_sourceUrl(sourceUrl)
+        self.set_predLang(predLang)
+        self.set_isString(isString)
+        self.set_metadata(metadata)
+        self.set_lexicon(lexicon)
 
     def __str__(self):
         return self.__repr__()
@@ -389,6 +393,22 @@ class NifContext(NifString):
             & super(NifBase, self).__eq__(other)
         )
 
+    @property
+    def metadata(self):
+        """
+        Returns the metadata of the context
+        """
+        if self._metadata is not None:
+            return self._metadata
+        elif self.graph is not None:
+            metadata = {}
+            for p, o in self.graph.predicate_objects(subject=self.uri):
+                if p in DC or p in DCTERMS:
+                    metadata[p] = o
+            return metadata
+        else:
+            return None
+
     @property
     def sourceUrl(self):
         """
@@ -396,8 +416,8 @@ class NifContext(NifString):
         """
         if self._sourceUrl is not None:
             return self._sourceUrl
-        elif self.referenceContext.in_memory_graph is not None:
-            for item in self.referenceContext.in_memory_graph.objects(subject=self.uri, predicate=NIF.sourceUrl):
+        elif self.graph is not None:
+            for item in self.graph.objects(subject=self.uri, predicate=NIF.sourceUrl):
                 return item
         else:
             return None
@@ -409,9 +429,12 @@ class NifContext(NifString):
         """
         if self._predLang is not None:
             return self._predLang
-        elif self.referenceContext.in_memory_graph is not None:
-            for item in self.referenceContext.in_memory_graph.objects(subject=self.uri, predicate=NIF.predLang):
-                return item
+        elif self.graph is not None:
+            for item in self.graph.objects(subject=self.uri, predicate=NIF.predLang):
+                if isinstance(item, Literal):
+                    return item.value
+                else:
+                    return item
         else:
             return None
 
@@ -422,8 +445,8 @@ class NifContext(NifString):
         """
         if self._isString is not None:
             return self._isString.value
-        elif self.referenceContext.in_memory_graph is not None:
-            for item in self.referenceContext.in_memory_graph.objects(subject=self.uri, predicate=NIF.isString):
+        elif self.graph is not None:
+            for item in self.graph.objects(subject=self.uri, predicate=NIF.isString):
                 return item.value
         else:
             return None
@@ -548,6 +571,23 @@ class NifContext(NifString):
         else:
             return None
 
+    @property
+    def lexicon(self):
+        """
+        Returns the lexicon
+        """
+        return self._lexicon
+
+
+    def set_lexicon(self, lexicon: URIRef = None):
+        """
+        Sets the lexicon base uri for lemmas
+        """
+        if lexicon is not None:
+            self._lexicon = lexicon
+        else:
+            self._lexicon = None
+
     def set_metadata(self, metadata: dict = None):
         """
         Sets the metadata of the context (a dict of predicates and objects)
@@ -588,7 +628,7 @@ class NifContext(NifString):
         """
         Sets the pages of the context (a list of NifPage)
         """
-        if pages is not None:
+        if pages is not None and pages != []:
             self._pages = deque(pages)
         else:
             self._pages = None
@@ -597,7 +637,7 @@ class NifContext(NifString):
         """
         Sets the paragraphs of the context (a list of NifParagraph)
         """
-        if paragraphs is not None:
+        if paragraphs is not None and paragraphs != []:
             self._paragraphs = deque(paragraphs)
         else:
             self._paragraphs = None
@@ -606,7 +646,7 @@ class NifContext(NifString):
         """
         Sets the phrases of the context (a list of NifPhrases)
         """
-        if phrases is not None:
+        if phrases is not None and phrases != []:
             self._phrases = deque(phrases)
         else:
             self._phrases = None
@@ -615,7 +655,7 @@ class NifContext(NifString):
         """
         Sets the sentences of the context (a list of NifSentence)
         """
-        if sentences is not None:
+        if sentences is not None and sentences != []:
             self._sentences = deque(sentences)
         else:
             self._sentences = None
@@ -630,8 +670,10 @@ class NifContext(NifString):
             else:
                 self._sentences.append(sentence)
 
-    def set_in_memory_graph(self, graph: Graph=None):
-        self.in_memory_graph = graph
+    def set_graph(self, graph: Graph = None):
+        self.graph = graph
+        if graph is not None:
+            self.load()
 
     def triples(self):
         """
@@ -675,107 +717,57 @@ class NifContext(NifString):
             for triple in super().triples():
                 yield triple
 
-    def load(self, graph: Graph = None, uri: URIRef = None):
+    def load(self):
         """
         Load a context from a graph (based on uri)
         """
-
-        self.set_uri(uri)
-        self.set_referenceContext(self)
-
-        if isinstance(graph.store, rdflib.plugins.stores.sparqlstore.SPARQLUpdateStore):
-            q = (
-                """
-            SELECT ?s ?p ?o
-            WHERE {
-                SERVICE <"""
-                + graph.store.query_endpoint
-                + """>
-                {
-                    ?s nif:referenceContext """
-                + uri.n3(graph.namespace_manager)
-                + """ .
-                    ?s ?p ?o .
-                }
-            }"""
-            )
-        else:
-            q = (
-                """
-            SELECT ?s ?p ?o
-            WHERE {
-                ?s nif:referenceContext """
-                + uri.n3(graph.namespace_manager)
-                + """ .
-                ?s ?p ?o .
-            }"""
-            )
-        results = graph.query(q)
-
-        in_memory_graph = Graph(store="SimpleMemory")
-        for s, p, o in results:
-            # necessary if data is read from http protocol
-            if isinstance(o, Literal) and o.datatype == XSD.string:
-                o = Literal(o.value.replace("\r\n", "\n"), datatype=XSD.string)
-            in_memory_graph.add((s, p, o))
-        self.set_in_memory_graph(in_memory_graph)
-
         sent_uris = natural_sort(
-            [s for s, _, _ in self.in_memory_graph.triples([None, RDF.type, NIF.Sentence])]
+            [s for s in self.graph.subjects(predicate=RDF.type, object=NIF.Sentence)]
         )
         page_uris = natural_sort(
-            [s for s, _, _ in self.in_memory_graph.triples([None, RDF.type, NIF.Page])]
+            [s for s in self.graph.subjects(predicate=RDF.type, object=NIF.Page)]
         )
         para_uris = natural_sort(
-            [s for s, _, _ in self.in_memory_graph.triples([None, RDF.type, NIF.Para])]
+            [s for s in self.graph.subjects(predicate=RDF.type, object=NIF.Para)]
         )
         phrase_uris = natural_sort(
-            [s for s, _, _ in self.in_memory_graph.triples([None, RDF.type, NIF.Phrase])]
+            [s for s in self.graph.subjects(predicate=RDF.type, object=NIF.Phrase)]
         )
         # for _, _, o in graph.triples((uri, NIF.beginIndex, None)):
         #     self.set_beginIndex(o)
         # for _, _, o in graph.triples((uri, NIF.endIndex, None)):
         #     self.set_endIndex(o)
-        # for _, _, o in graph.triples((uri, NIF.isString, None)):
+        # for o in self.graph.objects(subject=self.uri, predicate=NIF.isString):
         #     self.set_isString(o)
         # for _, _, o in graph.triples((uri, NIF.sourceUrl, None)):
         #     self.set_sourceUrl(o)
         # for _, _, o in graph.triples((uri, NIF.predLang, None)):
         #     self.set_predLang(o)
-        for _, _, o in graph.triples((uri, RDF.type, NIF.OffsetBasedString)):
-            self.set_URIScheme(o)
-        for _, _, o in graph.triples((uri, RDF.type, NIF.RFC5147String)):
-            self.set_URIScheme(o)
-
-        metadata = {}
-        for _, p, o in graph.triples((uri, None, None)):
-            if p in DC or p in DCTERMS:
-                metadata[p] = o
-        self.set_metadata(metadata)
-
-        self.set_Sentences(
-            [
-                NifSentence(URIScheme=self.URIScheme).load(
-                    graph=self.in_memory_graph, referenceContext=self, uri=sent_uri
-                )
-                for sent_uri in sent_uris
-            ]
-        )
+        for o in self.graph.objects(subject=self.uri, predicate=RDF.type):
+            if o == NIF.OffsetBasedString or o == NIF.RFC5147String:
+                self.set_URIScheme(o)
 
         # extract pages from graph
         self.set_Pages(
             [
-                NifPage(URIScheme=self.URIScheme).load(
-                    graph=self.in_memory_graph, referenceContext=self, uri=page_uri
+                NifPage(
+                    URIScheme=self.URIScheme,
+                    uri=page_uri,
+                    referenceContext=self,
+                    pageNumber=idx+1,
+                    graph=self.graph,
                 )
-                for page_uri in page_uris
+                for idx, page_uri in enumerate(page_uris)
             ]
         )
         # extract paragraphs from graph
         self.set_Paragraphs(
             [
-                NifParagraph(URIScheme=self.URIScheme).load(
-                    graph=self.in_memory_graph, referenceContext=self, uri=para_uri
+                NifParagraph(
+                    URIScheme=self.URIScheme,
+                    uri=para_uri,
+                    referenceContext=self,
+                    graph=self.graph,
                 )
                 for para_uri in para_uris
             ]
@@ -783,13 +775,59 @@ class NifContext(NifString):
         # extract phrases from graph
         self.set_Phrases(
             [
-                NifPhrase(URIScheme=self.URIScheme).load(
-                    graph=self.in_memory_graph, referenceContext=self, uri=phrase_uri
+                NifPhrase(
+                    URIScheme=self.URIScheme,
+                    uri=phrase_uri,
+                    referenceContext=self,
+                    graph=self.graph,
                 )
                 for phrase_uri in phrase_uris
             ]
         )
 
+        nifsentences = []
+        for sent_uri in sent_uris:
+            nifsentence = NifSentence(
+                URIScheme=self.URIScheme,
+                uri=sent_uri,
+                referenceContext=self,
+                graph=self.graph,
+            )
+
+            word_uris = list()
+            for s in self.graph.subjects(predicate=NIF.sentence, object=sent_uri):
+                if (s, RDF.type, NIF.Word) in self.graph:
+                    word_uris.append(s)
+            word_uris = natural_sort(word_uris)
+
+            # extract words from graph
+            words = OrderedDict()
+            for word_uri in word_uris:
+                words[word_uri] = NifWord(
+                    URIScheme=self.URIScheme,
+                    uri=word_uri,
+                    referenceContext=self.referenceContext,
+                    nifsentence=nifsentence,
+                    graph=self.graph,
+                )
+            nifsentence.set_Words(words.values())
+
+            # replace dependency uris by word objects
+            for word_idx, word in enumerate(words.values()):
+                word.set_dependency([words[dep] for dep in word.dependency])
+
+            words = nifsentence.words
+            if words is not None:
+                # replace nextWord and previousWord uris by word objects
+                for word_idx, word in enumerate(words):
+                    if word_idx > 0:
+                        word.set_previousWord(words[word_idx - 1])
+                    if word_idx < len(words) - 1:
+                        word.set_nextWord(words[word_idx + 1])
+
+            nifsentences.append(nifsentence)
+
+        self.set_Sentences(nifsentences)
         sentences = self.sentences
         if sentences is not None:
             for sent_idx, sentence in enumerate(sentences):
@@ -798,8 +836,6 @@ class NifContext(NifString):
                 if sent_idx < len(sentences) - 1:
                     sentence.set_nextSentence(sentences[sent_idx + 1])
 
-        return self
-
     def load_from_dict(self, stanza_dict: list = None):
         """
         Load a context from stanza dictionary
@@ -872,7 +908,7 @@ class NifContext(NifString):
                         sentence.set_nextSentence(sentences[sent_idx + 1])
                     if sent_idx > 0:
                         sentence.set_previousSentence(sentences[sent_idx - 1])
-
+        
 
 class NifStructure(NifString):
     """
@@ -897,6 +933,7 @@ class NifStructure(NifString):
         beginIndex: Union[Literal, int] = None,
         endIndex: Union[Literal, int] = None,
         referenceContext: NifContext = None,
+        graph: Graph = None,
     ):
         super().__init__(
             URIScheme=URIScheme,
@@ -904,6 +941,7 @@ class NifStructure(NifString):
             beginIndex=beginIndex,
             endIndex=endIndex,
             referenceContext=referenceContext,
+            graph=graph,
         )
 
     def triples(self):
@@ -951,6 +989,7 @@ class NifPhrase(NifStructure):
         PhraseType: str = None,
         nextPhrase: NifPhrase = None,
         previousPhrase: NifPhrase = None,
+        graph: Graph = None,
     ):
         super().__init__(
             URIScheme=URIScheme,
@@ -958,6 +997,7 @@ class NifPhrase(NifStructure):
             beginIndex=beginIndex,
             endIndex=endIndex,
             referenceContext=referenceContext,
+            graph=graph,
         )
         self.set_taIdentRef(taIdentRef)
         self.set_taClassRef(taClassRef)
@@ -997,6 +1037,10 @@ class NifPhrase(NifStructure):
         """
         if self._PhraseType is not None:
             return self._PhraseType
+        elif self.graph is not None:
+            for item in self.graph.objects(subject=self.uri, predicate=RDF.type):
+                if o in [TermOccurrence, EntityOccurrence]:
+                    return o
         else:
             return None
 
@@ -1007,6 +1051,11 @@ class NifPhrase(NifStructure):
         """
         if self._taIdentRef is not None:
             return self._taIdentRef
+        elif self.graph is not None:
+            for item in self.graph.objects(
+                subject=self.uri, predicate=ITSRDF.taIdentRef
+            ):
+                return item
         else:
             return None
 
@@ -1017,6 +1066,11 @@ class NifPhrase(NifStructure):
         """
         if self._taClassRef is not None:
             return self._taClassRef
+        elif self.graph is not None:
+            for item in self.graph.objects(
+                subject=self.uri, predicate=ITSRDF.taClassRef
+            ):
+                return item
         else:
             return None
 
@@ -1027,6 +1081,11 @@ class NifPhrase(NifStructure):
         """
         if self._taConfidence is not None:
             return float(self._taConfidence.value)
+        elif self.graph is not None:
+            for item in self.graph.objects(
+                subject=self.uri, predicate=ITSRDF.taConfidence
+            ):
+                return float(item.value)
         else:
             return None
 
@@ -1114,35 +1173,6 @@ class NifPhrase(NifStructure):
             for triple in super().triples():
                 yield triple
 
-    def load(
-        self,
-        graph: Graph = None,
-        referenceContext: NifContext = None,
-        uri: URIRef = None,
-    ):
-        """
-        Load a phrase from a graph (based on uri)
-        """
-        self.set_uri(uri)
-        self.set_referenceContext(referenceContext)
-        # for s, p, o in graph.triples((uri, NIF.beginIndex, None)):
-        #     self.set_beginIndex(o)
-        # for s, p, o in graph.triples((uri, NIF.endIndex, None)):
-        #     self.set_endIndex(o)
-        # for s, p, o in graph.triples((uri, NIF.anchorOf, None)):
-        #     self.set_anchorOf(o)
-        for s, p, o in graph.triples((uri, ITSRDF.taIdentRef, None)):
-            self.set_taIdentRef(o)
-        for s, p, o in graph.triples((uri, ITSRDF.taClassRef, None)):
-            self.set_taClassRef(o)
-        for s, p, o in graph.triples((uri, ITSRDF.taConfidence, None)):
-            self.set_taConfidence(o)
-        for s, p, o in graph.triples((uri, RDF.type, None)):
-            if o in [TermOccurrence, EntityOccurrence]:
-                self.set_PhraseType(o)
-
-        return self
-
 
 class NifSentence(NifStructure):
     """
@@ -1173,7 +1203,8 @@ class NifSentence(NifStructure):
         referenceContext: NifContext = None,
         nextSentence: Union[URIRef, str] = None,
         previousSentence: Union[URIRef, str] = None,
-        words: list[Union[NifWord, URIRef]] = None,
+        words: List[Union[NifWord, URIRef]] = None,
+        graph: Graph = None,
     ):
         super().__init__(
             URIScheme=URIScheme,
@@ -1181,6 +1212,7 @@ class NifSentence(NifStructure):
             beginIndex=beginIndex,
             endIndex=endIndex,
             referenceContext=referenceContext,
+            graph=graph,
         )
         self.set_nextSentence(nextSentence)
         self.set_previousSentence(previousSentence)
@@ -1319,46 +1351,6 @@ class NifSentence(NifStructure):
         # for s, p, o in graph.triples((uri, NIF.lastWord, None)):
         #     last_word = o
 
-        word_uris = list()
-        for s, _, _ in graph.triples((None, NIF.sentence, self.uri)):
-            if (s, RDF.type, NIF.Word) in graph:
-                word_uris.append(s)
-        word_uris = natural_sort(word_uris)
-
-        # extract words from graph
-        words = OrderedDict()
-        for word_uri in word_uris:
-            words[word_uri] = NifWord(URIScheme=self.URIScheme).load(
-                graph=graph,
-                referenceContext=self.referenceContext,
-                nifsentence=self,
-                uri=word_uri,
-            )
-        self.set_Words(words.values())
-
-        # replace dependency uris by word objects
-        for word_idx, word in enumerate(words.values()):
-            word.set_dependency([words[dep] for dep in word.dependency])
-
-        words = self.words
-        if words is not None:
-            # replace nextWord and previousWord uris by word objects
-            for word_idx, word in enumerate(words):
-                if word_idx > 0:
-                    word.set_previousWord(words[word_idx - 1])
-                if word_idx < len(words) - 1:
-                    word.set_nextWord(words[word_idx + 1])
-
-            # # check is firstWord in rdf data is first word in sentence object
-            # if self.firstWord.uri != first_word:
-            #     logging.error(
-            #         "Error: firstWord in rdf data not first word in sentence."
-            #     )
-
-            # # check is lastWord in rdf data is last word in sentence object
-            # if self.lastWord.uri != last_word:
-            #     logging.error("Error: lastWord in rdf data not last word in context.")
-
         return self
 
 
@@ -1385,6 +1377,7 @@ class NifParagraph(NifStructure):
         beginIndex: Union[Literal, int] = None,
         endIndex: Union[Literal, int] = None,
         referenceContext: NifContext = None,
+        graph: Graph = None,
     ):
         super().__init__(
             URIScheme=URIScheme,
@@ -1392,6 +1385,7 @@ class NifParagraph(NifStructure):
             beginIndex=beginIndex,
             endIndex=endIndex,
             referenceContext=referenceContext,
+            graph=graph,
         )
 
     def __str__(self):
@@ -1432,6 +1426,8 @@ class NifPage(NifStructure):
 
     :param endIndex: the end index in the context string
 
+    :param pageNumber: the page number of the object
+
     :param referenceContext: the context to which the string refers
 
     """
@@ -1442,7 +1438,9 @@ class NifPage(NifStructure):
         uri: Union[URIRef, str] = None,
         beginIndex: Union[Literal, int] = None,
         endIndex: Union[Literal, int] = None,
+        pageNumber: int = None,
         referenceContext: NifContext = None,
+        graph: Graph = None,
     ):
         super().__init__(
             URIScheme=URIScheme,
@@ -1450,7 +1448,9 @@ class NifPage(NifStructure):
             beginIndex=beginIndex,
             endIndex=endIndex,
             referenceContext=referenceContext,
+            graph=graph,
         )
+        self.set_pageNumber(pageNumber)
 
     def __str__(self):
         return self.__repr__()
@@ -1466,14 +1466,30 @@ class NifPage(NifStructure):
                 s += f'  anchorOf : {repr(self.anchorOf[0:1000]+"... ")}\n'
             else:
                 s += f"  anchorOf : {repr(self.anchorOf)}\n"
+        if self.pageNumber is not None and self.pageNumber != 0:
+            s += f"  pageNumber : {self.pageNumber}\n"
         return s
 
+    def set_pageNumber(self, pageNumber: int = None):
+        if pageNumber is not None and pageNumber != 0:
+            self._pageNumber = Literal(pageNumber, datatype=XSD.nonNegativeInteger)
+        else:
+            self._pageNumber = None
+
+    @property
+    def pageNumber(self):
+        if self._pageNumber is not None:
+            return self._pageNumber.value
+        else:
+            return None
+
     def triples(self):
         """
         Generates all the triples
         """
         if self.uri is not None:
             yield (self.uri, RDF.type, NIF.Page)
+            yield (self.uri, NIF.pageNumber, self._pageNumber)
             for triple in super().triples():
                 yield triple
 
@@ -1524,13 +1540,14 @@ class NifWord(NifStructure):
         endIndex: Union[Literal, int] = None,
         referenceContext: NifContext = None,
         nifsentence: NifSentence = None,
-        lemma: str = None,
+        lemma: Union[URIRef, str] = None,
         pos: list = None,
         morphofeats: list = None,
         dependency: list = None,
         dependencyRelationType: str = None,
         nextWord: str = None,
         previousWord: str = None,
+        graph: Graph = None,
     ):
         super().__init__(
             URIScheme=URIScheme,
@@ -1538,6 +1555,7 @@ class NifWord(NifStructure):
             beginIndex=beginIndex,
             endIndex=endIndex,
             referenceContext=referenceContext,
+            graph=graph,
         )
         self.set_nifsentence(nifsentence)
         self.set_lemma(lemma)
@@ -1595,10 +1613,16 @@ class NifWord(NifStructure):
         Returns the lemma of the word
         """
         if self._lemma is not None:
-            return self._lemma.value
-        elif self.referenceContext.in_memory_graph is not None:
-            for item in self.referenceContext.in_memory_graph.objects(subject=self.uri, predicate=NIF.lemma):
-                return item.value
+            if isinstance(self._lemma, Literal):
+                return self._lemma.value
+            else:
+                return self._lemma
+        elif self.graph is not None:
+            for item in self.graph.objects(subject=self.uri, predicate=NIF.lemma):
+                if isinstance(self._lemma, Literal):
+                    return item.value
+                else:
+                    return item
         else:
             return None
 
@@ -1609,8 +1633,10 @@ class NifWord(NifStructure):
         """
         if self._pos is not None:
             return self._pos
-        elif self.referenceContext.in_memory_graph is not None:
-            return [item for item in self.referenceContext.in_memory_graph.objects(subject=self.uri, predicate=NIF.pos)]
+        elif self.graph is not None:
+            return [
+                item for item in self.graph.objects(subject=self.uri, predicate=NIF.pos)
+            ]
         else:
             return None
 
@@ -1621,8 +1647,11 @@ class NifWord(NifStructure):
         """
         if self._morphofeats is not None:
             return self._morphofeats
-        elif self.referenceContext.in_memory_graph is not None:
-            return [item for item in self.referenceContext.in_memory_graph.objects(subject=self.uri, predicate=NIF.oliaLink)]
+        elif self.graph is not None:
+            return [
+                item
+                for item in self.graph.objects(subject=self.uri, predicate=NIF.oliaLink)
+            ]
         else:
             return []
 
@@ -1633,8 +1662,13 @@ class NifWord(NifStructure):
         """
         if self._dependency is not None:
             return self._dependency
-        elif self.referenceContext.in_memory_graph is not None:
-            return [item for item in self.referenceContext.in_memory_graph.objects(subject=self.uri, predicate=NIF.dependency)]
+        elif self.graph is not None:
+            return [
+                item
+                for item in self.graph.objects(
+                    subject=self.uri, predicate=NIF.dependency
+                )
+            ]
         else:
             return []
 
@@ -1645,8 +1679,10 @@ class NifWord(NifStructure):
         """
         if self._dependencyRelationType is not None:
             return self._dependencyRelationType.value
-        elif self.referenceContext.in_memory_graph is not None:
-            for item in self.referenceContext.in_memory_graph.objects(subject=self.uri, predicate=NIF.dependencyRelationType):
+        elif self.graph is not None:
+            for item in self.graph.objects(
+                subject=self.uri, predicate=NIF.dependencyRelationType
+            ):
                 return item.value
         else:
             return None
@@ -1677,12 +1713,16 @@ class NifWord(NifStructure):
         """
         self._nifsentence = nifsentence
 
-    def set_lemma(self, lemma: str = None):
+    def set_lemma(self, lemma: Union[URIRef, str] = None):
         """
         Sets the lemma of the word (a string)
         """
         if lemma is not None and lemma != "":
-            self._lemma = Literal(lemma, datatype=XSD.string)
+            if isinstance(lemma, URIRef):
+                self._lemma = lemma
+            else:
+                self._lemma = Literal(lemma, datatype=XSD.string)
+
         else:
             self._lemma = None
 
@@ -1777,7 +1817,13 @@ class NifWord(NifStructure):
                 yield triple
             yield (self.uri, NIF.anchorOf, Literal(self.anchorOf, datatype=XSD.string))
             if self.lemma is not None:
-                yield (self.uri, NIF.lemma, self._lemma)
+                if self.referenceContext.lexicon is not None:
+                    # prevent that uribaker converts this to underscore
+                    lemma = self._lemma.replace('"', "%22")
+                    lemma_uri = URIRef(iribaker.to_iri(str(self.referenceContext.lexicon)+lemma))
+                    yield (self.uri, NIF.lemma, lemma_uri)
+                else:
+                    yield (self.uri, NIF.lemma, self._lemma)
             if self.pos is not None and self._pos != []:
                 for pos in self._pos:
                     yield (self.uri, NIF.pos, pos)
@@ -1798,43 +1844,7 @@ class NifWord(NifStructure):
                 for dep in self._dependency:
                     yield (self.uri, NIF.dependency, dep.uri)
 
-    def load(
-        self,
-        graph: Graph = None,
-        referenceContext: NifContext = None,
-        nifsentence: NifSentence = None,
-        uri: URIRef = None,
-    ):
-        """
-        Load a word from a graph (based on uri of sentence)
-        """
-        self.set_uri(uri)
-        self.set_referenceContext(referenceContext)
-        self.set_nifsentence(nifsentence)
-        # for s, p, o in graph.triples((uri, NIF.beginIndex, None)):
-        #     self.set_beginIndex(o)
-        # for s, p, o in graph.triples((uri, NIF.endIndex, None)):
-        #     self.set_endIndex(o)
-        # for s, p, o in graph.triples((uri, NIF.anchorOf, None)):
-        #     self.set_anchorOf(o)
-        # for s, p, o in graph.triples((uri, NIF.nextWord, None)):
-        #     self.set_nextWord(o)
-        # for s, p, o in graph.triples((uri, NIF.previousWord, None)):
-        #     self.set_previousWord(o)
-        # for s, p, o in graph.triples((uri, NIF.lemma, None)):
-        #     self.set_lemma(o)
-        # for s, p, o in graph.triples((uri, NIF.pos, None)):
-        #     self.add_pos(o)
-        # for s, p, o in graph.triples((uri, NIF.oliaLink, None)):
-        #     self.add_morphofeat(o)
-        # for s, p, o in graph.triples((uri, NIF.dependency, None)):
-        #     self.add_dependency(o)
-        # for s, p, o in graph.triples((uri, NIF.dependencyRelationType, None)):
-        #     self.set_dependencyRelationType(o)
 
-        return self
-
-    
 class NifContextCollection(NifBase):
     """
     A NIF Context Collection
